package java.security;

import java.nio.ByteBuffer;
import sun.security.util.Debug;
import java.util.HashMap;

public class SecureClassLoader extends ClassLoader
{
    private final boolean initialized;
    private final HashMap<CodeSource, ProtectionDomain> pdcache;
    private static final Debug debug;
    
    protected SecureClassLoader(ClassLoader parent) {
        super(parent);
        this.pdcache = new HashMap(11);
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkCreateClassLoader();
        }
        this.initialized = true;
    }
    
    protected SecureClassLoader() {
        this.pdcache = new HashMap(11);
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkCreateClassLoader();
        }
        this.initialized = true;
    }
    
    protected final Class<?> defineClass(String name, byte[] b, int off, int len, CodeSource codeSource) {
        return super.defineClass(name, b, off, len, this.getProtectionDomain(codeSource));
    }
    
    protected final Class<?> defineClass(String name, ByteBuffer b, CodeSource codeSource) {
        return super.defineClass(name, b, this.getProtectionDomain(codeSource));
    }
    
    protected PermissionCollection getPermissions(CodeSource codeSource) {
        this.check();
        return new Permissions();
    }
    
    private ProtectionDomain getProtectionDomain(CodeSource codeSource) {
        if (codeSource == null) {
            return null;
        }
        ProtectionDomain protectionDomain = null;
        synchronized (this.pdcache) {
            protectionDomain = (ProtectionDomain)this.pdcache.get(codeSource);
            if (protectionDomain == null) {
                protectionDomain = new ProtectionDomain(codeSource, this.getPermissions(codeSource), this, null);
                this.pdcache.put(codeSource, protectionDomain);
                if (SecureClassLoader.debug != null) {
                    SecureClassLoader.debug.println(" getPermissions " + protectionDomain);
                    SecureClassLoader.debug.println("");
                }
            }
        }
        return protectionDomain;
    }
    
    private void check() {
        if (!this.initialized) {
            throw new SecurityException("ClassLoader object not initialized");
        }
        return;
    }
    
    static {
        debug = Debug.getInstance("scl");
        ClassLoader.registerAsParallelCapable();
    }
}
